/*

SmarterMailMon: Mail account monitor that can be used to watch for new 
messages in POP and IMAP accounts.

Copyright (C) 2007  SmarterTools

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Xml;

namespace SmarterMailMon
{
    public delegate void MailAccountStatusUpdateDelegate(object sender, MailAccountStatusUpdateEventArgs args);

    #region MailAccountStatusUpdateEventArgs

    public class MailAccountStatusUpdateEventArgs
    {
        public string accountName = "";
        public FolderInfo[] folderInfoArr = new FolderInfo[0];
        public bool success = false;

        public MailAccountStatusUpdateEventArgs()
        {
        }

        public MailAccountStatusUpdateEventArgs(string name, FolderInfo[] folderInfoArr, bool success)
        {
            this.accountName = name;
            this.folderInfoArr = folderInfoArr;
            this.success = success;
        }
    }

    #endregion

    public class MailAccount
    {
        public string name = "";
        public string serverAddress = "";
        public string userName = "";
        public string password = "";
        public string webMailUrl = "";
        public MailServerType serverType = MailServerType.Other;
        public AccountType accountType = AccountType.POP;

        public event MailAccountStatusUpdateDelegate statusUpdateEvent = null;

        private Thread statusUpdateThread = null;

        public override string ToString()
        {
            return name;
        }

        public string GetHashKey()
        {
            return name.ToLowerInvariant();
        }

        public static string GetHashKey(string accountName)
        {
            return accountName.ToLowerInvariant();
        }

        public void RequestStatusUpdate()
        {
            if (statusUpdateThread != null && statusUpdateThread.IsAlive)
                return;

            statusUpdateThread = new Thread(new ThreadStart(UpdateStatus));
            statusUpdateThread.Start();
        }

        private void UpdateStatus()
        {
            MailClient mailClient = new MailClient(this);
            FolderInfo[] folderInfoArr = new FolderInfo[0];
            bool success = false;

            try
            {
                folderInfoArr = mailClient.GetMessageCounts();
                success = true;
            }
            catch
            {
                // add logging
            }

            MailAccountStatusUpdateEventArgs eventArgs = 
                new MailAccountStatusUpdateEventArgs(name, folderInfoArr, success);

            if (statusUpdateEvent != null)
                statusUpdateEvent(this, eventArgs);
        }

        public void KillUpdate()
        {
            if (statusUpdateThread != null && statusUpdateThread.IsAlive)
            {
                try
                {
                    statusUpdateThread.Abort();
                }
                catch { }
            }
        }

        #region Save and Load Functions

        public void SaveToXml(XmlWriter writer)
        {
            string encodedPassword = Convert.ToBase64String(Encoding.Default.GetBytes(password));

            writer.WriteStartElement("MailAccount");

            writer.WriteElementString("name", name);
            writer.WriteElementString("accountType", accountType.ToString());
            writer.WriteElementString("serverAddress", serverAddress);
            writer.WriteElementString("serverType", serverType.ToString());
            writer.WriteElementString("userName", userName);
            writer.WriteElementString("password", encodedPassword);
            writer.WriteElementString("webMailUrl", webMailUrl);

            writer.WriteEndElement();
        }

        public static MailAccount LoadFromXml(XmlReader reader)
        {
            MailAccount account = new MailAccount();

            try
            {
                string var = "";
                while (reader.Read())
                {
                    if (reader.NodeType == XmlNodeType.EndElement && reader.Name.ToLowerInvariant() == "mailaccount")
                        break;
                    else if (reader.NodeType == XmlNodeType.Element)
                        var = reader.Name;
                    else if (reader.NodeType == XmlNodeType.Text)
                    {
                        switch (var.ToLowerInvariant())
                        {
                            case "name":            account.name = reader.Value;            break;
                            case "serveraddress":   account.serverAddress = reader.Value;   break;
                            case "username":        account.userName = reader.Value;        break;
                            case "webmailurl":      account.webMailUrl = reader.Value;      break;
                            case "accounttype":
                                account.accountType = (AccountType)Enum.Parse(typeof(AccountType), reader.Value);
                                break;
                            case "servertype":
                                account.serverType = (MailServerType)Enum.Parse(typeof(MailServerType), reader.Value);
                                break;
                            case "password":
                                string passwordEncoded = reader.Value;
                                account.password = Encoding.Default.GetString(Convert.FromBase64String(passwordEncoded));
                                break;
                        }
                    }
                }
            }
            catch
            {
                return null;
            }

            return account;
        }

        #endregion
    }
}
