/*

SmarterMailMon: Mail account monitor that can be used to watch for new 
messages in POP and IMAP accounts.

Copyright (C) 2007  SmarterTools

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Xml;

namespace SmarterMailMon
{
    public delegate void MailAccountCollectionUpdateDelegate(object sender);

    public class MailAccountCollection
    {
        public static MailAccountCollection accountCollection = new MailAccountCollection();

        public event MailAccountCollectionUpdateDelegate collectionUpdateEvent = null;

        private List<MailAccount> accountList = new List<MailAccount>();
        private object accountListLockObj = new object();
        private string accountListFilePath = "accountList.xml";

        public MailAccount[] MailAccounts
        {
            get
            {
                MailAccount[] returnArr;
                lock (accountListLockObj)
                {
                    returnArr = accountList.ToArray();
                }
                return returnArr;
            }
        }

        public MailAccountCollection()
        {
            LoadFromXml();
        }

        public void AddAccount(MailAccount account)
        {
            lock (accountListLockObj)
            {
                accountList.Add(account);
                SaveToXml();
            }

            if (collectionUpdateEvent != null)
                collectionUpdateEvent(this);
        }

        public void UpdateAccount(MailAccount account)
        {
            UpdateAccount(account.name, account.accountType, account.serverAddress, account.serverType, 
                account.userName, account.password, account.webMailUrl);
        }

        public void UpdateAccount(string name, AccountType accountType, string serverAddress, MailServerType serverType, 
            string userName, string password, string webMailUrl)
        {
            MailAccount account = GetAccount(name);

            if (account == null)
                return;

            account.name = name;
            account.accountType = accountType;
            account.serverAddress = serverAddress;
            account.serverType = serverType;
            account.webMailUrl = webMailUrl;
            account.userName = userName;
            account.password = password;

            SaveToXml();

            if (collectionUpdateEvent != null)
                collectionUpdateEvent(this);
        }

        public void RemoveAccount(MailAccount account)
        {
            RemoveAccount(account.name);
        }

        public void RemoveAccount(string name)
        {
            lock (accountListLockObj)
            {
                string nameLow = name.ToLowerInvariant().Trim();

                for (int i = accountList.Count - 1; i >= 0; i--)
                {
                    string tempNameLow = accountList[i].name.ToLowerInvariant().Trim();
                    if (nameLow == tempNameLow)
                        accountList.RemoveAt(i);
                }

                SaveToXml();
            }

            if (collectionUpdateEvent != null)
                collectionUpdateEvent(this);
        }

        public MailAccount GetAccount(string name)
        {
            lock (accountListLockObj)
            {
                string nameLow = name.ToLowerInvariant().Trim();
                for (int i = accountList.Count - 1; i >= 0; i--)
                {
                    string tempNameLow = accountList[i].name.ToLowerInvariant().Trim();
                    if (nameLow == tempNameLow)
                        return accountList[i];
                }
            }

            return null;
        }

        public bool Exists(string name)
        {
            MailAccount tempMailAccount = GetAccount(name);
            return tempMailAccount != null;
        }

        #region Save and Load Functions

        public void SaveToXml()
        {
            lock (accountListLockObj)
            {
                using (XmlTextWriter writer = new XmlTextWriter(accountListFilePath, System.Text.Encoding.UTF8))
                {
                    writer.Formatting = Formatting.Indented;

                    writer.WriteStartDocument();
                    writer.WriteStartElement("AccountList");

                    foreach (MailAccount accountItem in accountList)
                        accountItem.SaveToXml(writer);

                    writer.WriteEndElement();
                    writer.WriteEndDocument();
                }
            }
        }

        public void LoadFromXml()
        {
            lock (accountListLockObj)
            {
                try
                {
                    using (XmlTextReader reader = new XmlTextReader(accountListFilePath))
                    {
                        while (reader.Read())
                        {
                            if (reader.NodeType == XmlNodeType.Element)
                            {
                                string elementName = reader.Name.ToLowerInvariant();
                                if (elementName == "mailaccount")
                                {
                                    MailAccount account = MailAccount.LoadFromXml(reader);
                                    if (account != null)
                                    {
                                        accountList.Add(account);
                                    }                                        
                                }
                            }
                        }
                    }
                }
                catch { }
            }
        }

        #endregion
    }
}
