/*

SmarterMailMon: Mail account monitor that can be used to watch for new 
messages in POP and IMAP accounts.

Copyright (C) 2007  SmarterTools

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/

using System;
using System.Collections.Generic;
using System.IO;
using System.Net.Sockets;
using System.Text;

namespace SmarterMailMon
{
    public class MailClient
    {
        public MailAccount mailAccount;
        public int port = 110;

        private TcpClient tcpClient = null;
        private StreamReader streamReader = null;

        public MailClient(MailAccount mailAccount)
        {
            this.mailAccount = mailAccount;

            port = mailAccount.accountType == AccountType.IMAP ? 143 : 110;
        }

        public FolderInfo[] GetMessageCounts()
        {
            if (mailAccount.accountType == AccountType.IMAP)
                return GetImapMessageCounts();
            return GetPopMessageCount();
        }

        private FolderInfo[] GetImapMessageCounts()
        {
            List<FolderInfo> folderInfoList = new List<FolderInfo>();
            int cmdCounter = 1;

            using (tcpClient = new TcpClient(mailAccount.serverAddress, port))
            {
                using (streamReader = new StreamReader(tcpClient.GetStream()))
                {
                    tcpClient.SendTimeout = 30000;
                    tcpClient.ReceiveTimeout = 30000;

                        // read initial connection response
                    streamReader.ReadLine();

                        // login
                    string loginCommand = string.Format("{0} LOGIN {1} {2}\r\n", 
                        cmdCounter++, mailAccount.userName, mailAccount.password);
                    SendIMAPCommand(loginCommand);

                        // list folders
                    string listCommand = string.Format("{0} LIST \"\" \"*\"\r\n", cmdCounter++);
                    string[] responseLines = SendIMAPCommand(listCommand);
                    
                        // build a list of folders from the response to the list command
                    foreach (string responseLineItem in responseLines)
                    {
                        string parseStr = responseLineItem.Split(')')[1].Trim();
                        parseStr = parseStr.Substring(parseStr.IndexOf(' ') + 1).Trim();
                        if (parseStr.StartsWith("\"") && parseStr.EndsWith("\""))
                            parseStr = parseStr.Substring(1, parseStr.Length - 2);
                        folderInfoList.Add(new FolderInfo(parseStr, 0));
                    }

                        // Get unread message counts
                    foreach (FolderInfo folderInfoItem in folderInfoList)
                    {
                        string statusCommand = string.Format("{0} STATUS \"{1}\" (UNSEEN)\r\n", cmdCounter++, folderInfoItem.name);
                        responseLines = SendIMAPCommand(statusCommand);

                        string parseLine = responseLines[0];
                        foreach (string lineItem in responseLines)
                            if (lineItem.IndexOf(folderInfoItem.name) > -1)
                            {
                                parseLine = lineItem;
                                break;
                            }

                        string[] parseSplit = parseLine.Replace(")", "").Trim().Split(' ');
                        folderInfoItem.count = int.Parse(parseSplit[parseSplit.Length - 1]);
                    }
                
                        // logout
                    string logoutCommand = string.Format("{0} LOGOUT\r\n", cmdCounter++);
                    SendIMAPCommand(logoutCommand);
                }
            }
            

            return folderInfoList.ToArray();
        }

        private FolderInfo[] GetPopMessageCount()
        {
            List<FolderInfo> folderInfoList = new List<FolderInfo>();
            FolderInfo folderInfo = new FolderInfo("Inbox", 0);

            using (tcpClient = new TcpClient(mailAccount.serverAddress, port))
            {
                using (streamReader = new StreamReader(tcpClient.GetStream()))
                {
                    // read the response to the initial connection
                    streamReader.ReadLine();

                        // login
                    string userCommand = string.Format("USER {0}\r\n", mailAccount.userName);
                    SendPOPCommand(userCommand);
                    string passwordCommand = string.Format("PASS {0}\r\n", mailAccount.password);
                    SendPOPCommand(passwordCommand);

                        // send stat command
                    string statResponseLine = SendPOPCommand("STAT\r\n");

                        // quit
                    SendPOPCommand("QUIT\r\n");

                    folderInfo.count = int.Parse(statResponseLine.Split(' ')[1]);
                    folderInfoList.Add(folderInfo);
                }
            }

            return folderInfoList.ToArray();
        }

        private string[] SendIMAPCommand(string command)
        {
                // anything we send needs to end with a carriage return + line feed sequence
            if (!command.EndsWith("\r\n"))
                command += "\r\n";

            byte[] bytesToSend = Encoding.ASCII.GetBytes(command);
            tcpClient.Client.Send(bytesToSend);

            List<string> responseList = new List<string>();
            while (true)
            {
                string line = streamReader.ReadLine();
                if (!line.StartsWith("*") && line.Split(' ')[1] == "OK")
                    break;
                responseList.Add(line);
            }

            return responseList.ToArray();
        }

        private string SendPOPCommand(string command)
        {
                // anything we send needs to end with a carriage return + line feed sequence
            if (!command.EndsWith("\r\n"))
                command += "\r\n";

            byte[] bytesToSend = Encoding.ASCII.GetBytes(command);
            tcpClient.Client.Send(bytesToSend);

            string responseLine = streamReader.ReadLine();
            
            return responseLine;
        }
    }
}
