/*

SmarterMailMon: Mail account monitor that can be used to watch for new 
messages in POP and IMAP accounts.

Copyright (C) 2007  SmarterTools

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace SmarterMailMon
{
    public partial class Form1 : Form
    {
        private delegate void UpdateAccountTreeViewDelegate();
        private delegate void RefreshAccountListDelegate();

        private Dictionary<string, int> attachedAccountDict = new Dictionary<string, int>();
        private Queue<MailAccountStatusUpdateEventArgs> updateQueue = new Queue<MailAccountStatusUpdateEventArgs>();
        private object updateQueueLockObj = new object();
        private System.Timers.Timer updateTimer = new System.Timers.Timer();
        private int iniLeft = -1, iniTop = -1;
        private Rectangle bounds = Rectangle.Empty;

        public Form1()
        {
            InitializeComponent();

            updateTimer = new System.Timers.Timer(10 * 60000);	// default to every 10 minutes
            updateTimer.Elapsed += new System.Timers.ElapsedEventHandler(updateTimer_Elapsed);
            updateTimer.Start();

            LoadINI();			// load the initial state of this app

            RefreshAccountList();
            MailAccountCollection.accountCollection.collectionUpdateEvent += new MailAccountCollectionUpdateDelegate(accountCollection_collectionUpdateEvent);
        }

        void updateTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            try
            {
                Invoke(new RefreshAccountListDelegate(RefreshAccountList));
            }
            catch { }
        }

        #region Load and Write ini file

        private void LoadINI()
        {
            if (!File.Exists("app.ini"))
                return;

            int width, height, updateInterval = 10;

            try
            {
                using (BinaryReader br = new BinaryReader(new FileStream("app.ini", FileMode.Open, FileAccess.Read, FileShare.Read)))
                {
                    string version = br.ReadString();

                    switch (version)
                    {
                        case "1.0":
                            iniTop = br.ReadInt32();
                            iniLeft = br.ReadInt32();
                            width = br.ReadInt32();
                            height = br.ReadInt32();
                            try
                            {
                                updateInterval = Convert.ToInt32(br.ReadString());
                            }
                            catch { }

                            this.Size = new System.Drawing.Size(width, height);

                            txtRefreshFrequency.Text = updateInterval.ToString();
                            SetTimer(updateInterval);

                            break;
                    }
                }
            }
            catch {}
        }

        private void WriteINI()
        {
            if (bounds == Rectangle.Empty)
                bounds = DesktopBounds;

            try
            {
                using (BinaryWriter bw = new BinaryWriter(new FileStream("app.ini", FileMode.OpenOrCreate, FileAccess.Write, FileShare.None)))
                {
                    bw.Write("1.0");
                    bw.Write(bounds.Top);
                    bw.Write(bounds.Left);
                    bw.Write(bounds.Width);
                    bw.Write(bounds.Height);
                    bw.Write(txtRefreshFrequency.Text);
                }
            }
            catch {}
        }

        private void SetTimer(int timeInMinutes)
        {
            if (timeInMinutes > 0)
                updateTimer.Interval = timeInMinutes * 60000;		// convert to milliseconds
        }

        #endregion

        private void accountCollection_collectionUpdateEvent(object sender)
        {
            try
            {
                Invoke(new RefreshAccountListDelegate(RefreshAccountList));
            }
            catch { }
        }

        private void RefreshAccountList()
        {
            MailAccount[] mailAccounts = MailAccountCollection.accountCollection.MailAccounts;

                // clear out the accounts tree view and the updateQueue
            tvAccounts.Nodes.Clear();
            lock (updateQueueLockObj)
            {
                updateQueue.Clear();
            }

                // add a top level node for each mail account and show a question mark for the total message count
            foreach (MailAccount accountItem in mailAccounts)
            {
                string nodeText = string.Format("{0} (?)", accountItem.name);
                tvAccounts.Nodes.Add(nodeText);
            }

                // request an update for each account
            foreach (MailAccount accountItem in mailAccounts)
            {
                    // keep track of those mail accounts whose update events we have attached to
                string accountHashKey = accountItem.GetHashKey();
                if (!attachedAccountDict.ContainsKey(accountHashKey))
                {
                    accountItem.statusUpdateEvent += new MailAccountStatusUpdateDelegate(accountItem_statusUpdateEvent);
                    attachedAccountDict.Add(accountHashKey, 0);
                }

                accountItem.RequestStatusUpdate();
            }
        }

        private void UpdateAccountsTreeView()
        {
            lock (updateQueueLockObj)
            {
                while (updateQueue.Count > 0)
                {
                    MailAccountStatusUpdateEventArgs updateArgs = updateQueue.Dequeue();
                    UpdateAccountNode(updateArgs);
                }
            }            
        }

        private void UpdateAccountNode(MailAccountStatusUpdateEventArgs updateArgs)
        {
            TreeNode accountRootNode = GetAccountsRootNode(updateArgs.accountName);
            if (accountRootNode == null)
                return;

            WriteAccountNode(accountRootNode, updateArgs);
        }

        private void WriteAccountNode(TreeNode accountRootNode, MailAccountStatusUpdateEventArgs updateArgs)
        {
            int totalMessageCount = 0;

                // Clear any nodes under the accounts root node if there are any.  There shouldn't be.
            accountRootNode.Nodes.Clear();

                // Write out a node for each folder in the account
            foreach (FolderInfo folderInfoItem in updateArgs.folderInfoArr)
            {
                accountRootNode.Nodes.Add(folderInfoItem.ToString());
                totalMessageCount += folderInfoItem.count;
            }

                // Update the text on the root node to reflect the total message count
            accountRootNode.Text = string.Format("{0} ({1})", updateArgs.accountName, totalMessageCount);

            accountRootNode.Expand();
        }

        private TreeNode GetAccountsRootNode(string accountName)
        {
            string accountNameLow = accountName.ToLowerInvariant();

                // Check the first level of nodes for the matching account.  Return if found.
            foreach (TreeNode nodeItem in tvAccounts.Nodes)
            {
                string nodeText = nodeItem.Text;
                int splitIndex = nodeText.LastIndexOf('(') - 1;
                if (splitIndex <= 0)
                    continue;

                    // Check if we found the node matching the account
                string accountNameFromNode = nodeText.Substring(0, splitIndex).Trim();
                if (accountNameFromNode.ToLowerInvariant() == accountNameLow)
                    return nodeItem;
            }

            return null;    // Failed to find the node corresponding to the given account name
        }

        private void accountItem_statusUpdateEvent(object sender, MailAccountStatusUpdateEventArgs args)
        {
            try
            {
                // don't update if the update was not successfull
                if (!args.success)
                    return;

                lock (updateQueueLockObj)
                {
                    updateQueue.Enqueue(args);
                }

                // tell the form that it has a new update to process
                Invoke(new UpdateAccountTreeViewDelegate(UpdateAccountsTreeView));
            }
            catch { }
        }

        private void btnEditAccounts_Click(object sender, EventArgs e)
        {
            EditAccounts editAccountsForm = new EditAccounts();
            editAccountsForm.ShowDialog(this);
        }

        private void tvAccounts_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {

        }

        private void txtRefreshFrequency_TextChanged(object sender, EventArgs e)
        {
            int interval = (int)(updateTimer.Interval / 60000);
            int newInterval = 0;

            try
            {
                newInterval = Convert.ToInt32(txtRefreshFrequency.Text);
                SetTimer(newInterval);
            }
            catch {}
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            updateTimer.Stop();

                // detach from the events we are attached to
            MailAccount[] accounts = MailAccountCollection.accountCollection.MailAccounts;
            foreach (MailAccount accountItem in accounts)
            {
                try
                {
                    accountItem.statusUpdateEvent -= new MailAccountStatusUpdateDelegate(accountItem_statusUpdateEvent);
                    accountItem.KillUpdate();
                }
                catch { }
            }
            MailAccountCollection.accountCollection.collectionUpdateEvent -= new MailAccountCollectionUpdateDelegate(accountCollection_collectionUpdateEvent);            

                // save main window location
            WriteINI();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            if (iniLeft != -1 && iniTop != -1)
                SetDesktopLocation(iniLeft, iniTop);
            bounds = DesktopBounds;
        }

        private void Form1_LocationChanged(object sender, EventArgs e)
        {
            if (WindowState != FormWindowState.Minimized)
                bounds = DesktopBounds;
        }

        private void Form1_SizeChanged(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
                this.Visible = false;
            else
                bounds = DesktopBounds;
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            RefreshAccountList();
        }
    }
}